class CalcofiCoordConverter {
    private double[] crds;
    private String proj;
    
    public CalcofiCoordConverter (double x,
            double y, String projection){
        if (!projection.equals("longlat") &&
            !projection.equals("calcofi")) {
            throw new IllegalArgumentException(
                "projection must be either " +
                "'longlat' or 'calcofi'");
        }
        proj = projection;
        
        if (proj.equals("longlat") && 
            x > 180.0) x = (x - 360.0) * -1.0;

        crds = new double[2];
        crds[0] = x;
        crds[1] = y;
    }
    
    public double[] getCoords() {
        return crds;
    }
    
    public String getProjection() {
        return proj;
    }
    
    private double toMercator(double lat) {
        double y = 
            Math.toDegrees(Math.log(
                Math.tan(Math.toRadians(
                45.0 + lat / 2))) - 
                0.00676866 * Math.sin(
                Math.toRadians(lat)));
        return y;
    }
    
    private double inverseMercator(
            double mercLat) {
        double approxLat = mercLat;
        for (int i = 1; i < 4; i++) {
            approxLat = 2 * (Math.atan(
                Math.exp(Math.toRadians(mercLat) + 
                0.00676866 * Math.sin(
                Math.toRadians(approxLat)))) *  
                180 / Math.PI - 45);
        }
        return approxLat;
    }
    
    public void reproject(String projection) {
        if (projection.equals("calcofi") &&
                !proj.equals("calcofi")) {
            double lon = crds[0];
            double lat = crds[1];
            if (lon > 180.0) lon = (
                lon - 360.0) * -1.0;
            // assume pos in the western hemisphere

            if (lon < 0) lon = lon * -1;
            double L1 = (toMercator(lat) -
                toMercator(34.15)) * 
                Math.tan(Math.toRadians(30));
            double L2 = lon - L1 - 121.15;
            double mercRefLatitude = L2 *
                Math.cos(Math.toRadians(30)) * 
                Math.sin(Math.toRadians(30)) +
                toMercator(lat);
            double refLatitude = inverseMercator(
                mercRefLatitude);
            // line
            crds[0] = 80 - (refLatitude - 34.15) *
                5 / Math.cos(Math.toRadians(30));
            // station 
            crds[1] = 60 + (refLatitude - lat) *
                15 / Math.sin(Math.toRadians(30));

        } else if (projection.equals("longlat") &&
                !proj.equals("longlat")) {
            double line = crds[0];
            double station = crds[1];
            double refLatitude = 34.15 -  0.2 *
                (line - 80) * Math.cos(
                Math.toRadians(30));
            double lat = refLatitude -
                (station - 60) * Math.sin(
                Math.toRadians(30)) / 15;
            double L1 = (toMercator(lat) -
                toMercator(34.15)) * 
                Math.tan(Math.toRadians(30));
            double L2 = (toMercator(refLatitude) -
                toMercator(lat)) / (Math.cos(
                Math.toRadians(30)) * Math.sin(
                Math.toRadians(30)));
            crds[0] = -1 * (L1 + L2 + 121.15);
            crds[1] = lat;
        }
        proj = projection;
    }
}
